//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   MQueueMsg
//
// description: A SystemVerilog Class to provide an abstraction of the
// MockTurtle core's queue message.
//
//------------------------------------------------------------------------------
// Copyright CERN 2018
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`ifndef __MT_MQUEUE_MSG_INCLUDED
 `define __MT_MQUEUE_MSG_INCLUDED

class MQueueMsg;
   int core;
   int slot;
   uint32_t header[$];
   uint32_t data[$];

   function new (int core = 0, int slot = 0);
      this.core   = core;
      this.slot   = slot;
      this.header = {};
      this.data   = {};
   endfunction // new

   function string tostring ();
      uint32_t app_id;
      uint32_t flags;
      uint32_t msg_id;
      uint32_t len;
      uint32_t sync_id;
      uint32_t seq;
      string hdr, dat;

      app_id  = this.header[0] & 32'h0000ffff;
      flags   = (this.header[0] & 32'h00ff0000) >> 16;
      msg_id  = (this.header[0] & 32'hff000000) >> 24;
      len     = this.header[1] & 32'h0000ffff;
      sync_id = (this.header[1] & 32'hffff0000) >> 16;
      seq     = this.header[2];

      hdr = $sformatf ("APP_ID: 0x%4x, MSG_ID: 0x%2x, FLAGS: 0x%2x, LEN: %0d, SYNC_ID: 0x%4x, SEQ: %0d",
		       app_id, msg_id, flags, len, sync_id, seq);

      dat = " PAYLOAD:";
      for (int i = 0; i < len; i++) begin
	 dat = {dat, $sformatf(" 0x%8x", this.data[i])};
      end

      return {hdr, dat};
   endfunction // tostring

endclass // MQueueMsg

`endif //  `ifndef __MT_MQUEUE_MSG_INCLUDED
