/*
 * Copyright (C) 2013-2016 CERN (www.cern.ch)
 * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
 * Author: Federico Vaga <federico.vaga@cern.ch>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#include <stdio.h>
#include <stdint.h>
#include <string.h>

#include "mockturtle-rt.h"


/**
 * It sends a character to the UART interface
 * @param[in] c the character to sent
 * @return 0 on success (for the time being it does not fail)
 */
int putchar(int c)
{
	lr_writel(c, MT_CPU_LR_REG_UART_CHR);

	return 0;
}


/**
 * It sends a string over the serial interface
 * @param[in] p string to send
 * @return number of sent characters
 *
 * The function does not add any special character.
 * If you neeed the new-line or the carriage-return
 * you have to add them to your strings.
 */
int puts(const char *p)
{
	const char *p_orig = p;
	char c;

	while ((c = *(p++)))
		putchar(c);

	return p - p_orig;
}


/**
 * It prints on the serial console the given message
 * @param[in] msg a mock turtle message
 */
void pr_message(struct trtl_fw_msg *msg)
{
	struct trtl_hmq_header *h = msg->header;
	uint32_t *d = msg->payload;
	int i;

	if (!HAS_MOCKTURTLE_LIBRARY_PRINT_MESSAGE_ENABLE)
		return;

	pp_printf("Message\n\r");
	pp_printf("\tapp_id 0x%04"PRIx16" | flags   0x%02"PRIx8" | msg_id 0x%02"PRIx8"\r\n",
		 h->rt_app_id, h->flags, h->msg_id);
	pp_printf("\tlen 0x%04"PRIx16" | sync_id 0x%04"PRIx16"\r\n",
		 h->len, h->sync_id);
	pp_printf("\tseq 0x%08"PRIx32"\r\n",
		 h->seq);

	for (i = 0; i < h->len; i++)
		pp_printf("%s: data[%d] = 0x%"PRIx32"\n\r", __func__, i , d[i]);
}
